/**

  The Initial Developer of the Original Code is
  Matthieu  - http://www.programmation-facile.com/
  Portions created by the Initial Developer are Copyright (C) 2013
  the Initial Developer. All Rights Reserved.

  Contributor(s) :

 */



/**
 * Le Framework de jeux JavaScript Phaser
 *

Phaser v2.0.5 - http://phaser.io - @photonstorm - (c) 2014 Photon Storm Ltd. 
https://github.com/photonstorm/phaser
http://examples.phaser.io/

 */

document.addEventListener("DOMContentLoaded", Main, false); // appel au chargement de la page


/**
 * Fonction principale
 * Appelée au chargement de la page
 * 
 */
function Main()
{
    console.log("Main");

    // creation de la zone de jeu - API Canvas
    var game = new Phaser.Game(400, 490, Phaser.AUTO, 'idGameDiv');


    /**
     * Construction du moteur du jeu
     * Création et mise en place de tous les éléments du jeu
     * 
     * @type {[type]}
     */
    var mainState = 
    {

        /**
         * préchargement des éléments du jeu
         * 
         * @return {[type]} [description]
         */
        preload: function()
        { 
            game.stage.backgroundColor = '#71c5cf';// la couleur de fond du jeu

            game.load.image('bird', 'assets/bird-hero.gif');  // les éléments graphiques avec leurs identifiants
            game.load.image('pipe', 'assets/bird-pipe.png'); 
            game.load.audio('jump', 'assets/jump.wav');       // la musique du jeu lors d'un saut
        },


        /**
         * Création de la scène du jeu,
         * mise en place des graphismes...
         * 
         * @return {[type]} [description]
         */
        create: function() 
        { 
            game.physics.startSystem(Phaser.Physics.ARCADE);

            this.pipes = game.add.group();
            this.pipes.enableBody = true;
            this.pipes.createMultiple(20, 'pipe');  
            this.timer = this.game.time.events.loop(1500, this.addRowOfPipes, this);           

            this.bird = this.game.add.sprite(100, 245, 'bird');
            game.physics.arcade.enable(this.bird);
            this.bird.body.gravity.y = 1000; 

            this.bird.anchor.setTo(-0.2, 0.5);// définit la position du heros
        
            // écouteur sur la barre espace du clavier
            var spaceKey = this.game.input.keyboard.addKey(Phaser.Keyboard.SPACEBAR);
            spaceKey.onDown.add(this.jump, this); 

            // lorsque le joueur clique sur la sourie
            this.game.input.onDown.add(this.jump, this);

            // le score du joueur
            this.score = 0;
            this.labelScore = this.game.add.text(20, 20, "0", { font: "33px Arial", fill: "#ffffff" });  

            this.jumpSound = this.game.add.audio('jump');// ajout d'un son à chaque saut
        },


        /**
         * fonction appelée en continue pendant le jeu
         * 
         * @return {[type]} [description]
         */
        update: function() 
        {
            // si le joueur sort de l'écran, relance le jeu
            if (this.bird.inWorld == false)
                this.restartGame(); 

            // gestion de la collision dans le jeu : entre le héros et les tuyaux, appelle la fonction hitPipe si collision
            game.physics.arcade.overlap(this.bird, this.pipes, this.hitPipe, null, this); 

            // rotation du heros plus lente à partir d'une certaine valeur
            if (this.bird.angle < 20)
                this.bird.angle += 1;     
        },


        /**
         * Le saut du héros
         * 
         * @return {[type]} [description]
         */
        jump: function()
        {
            // si le héros est mort, pas de saut, sortie de la fonction
            if (this.bird.alive == false)
                return; 

            this.bird.body.velocity.y = -350;

            game.add.tween(this.bird).to({angle: -20}, 100).start();// animation du saut avec une tween sur l'angle de rotation

            this.jumpSound.play();// lecture du son "saut"
        },


        /**
         * Collision effectuée entre le héross et les pipes / tuyaux du jeu
         * Lorsque le héro est mort...
         * 
         * @return {[type]} [description]
         */
        hitPipe: function()
        {
            // le héros a touché un tuyau dans le jeu, jeu terminé, sortie de la fonction
            if (this.bird.alive == false)
                return;
                
            this.bird.alive = false;// mets la propriété vie du héros à false, il est mort

            this.game.time.events.remove(this.timer);// empêche l'apparition de novueau tuyaux, fin du jeu
        
            // stop le mouvement de tous les tuyaux
            this.pipes.forEachAlive(function(p)
            {
                p.body.velocity.x = 0;
            }, this);

        },


        /**
         * Redémarre le jeu
         * 
         * @return {[type]} [description]
         */
        restartGame: function()
        {
            game.state.start('main');
        },


        /**
         * Création d'un tuyau selon les paramètres d'entrées
         * 
         * @param {[type]} x [description]
         * @param {[type]} y [description]
         */
        addOnePipe: function(x, y)
        {
            var pipe = this.pipes.getFirstDead();

            pipe.reset(x, y);
            pipe.body.velocity.x = -200;  
            pipe.checkWorldBounds = true;
            pipe.outOfBoundsKill = true;
        },


        /**
         * Gestion de la création des tuyaux (position et taille)
         * 
         */
        addRowOfPipes: function() 
        {
            var hole = Math.floor(Math.random()*5)+1;
            
            for (var i = 0; i < 8; i++)
                if (i != hole && i != hole +1) 
                    this.addOnePipe(400, i*60+10);   
        
            this.score += 1;
            this.labelScore.text = this.score;  
        },

    };

    game.state.add('main', mainState); // ajoute le "moteur" dans le jeu
    game.state.start('main'); // lance le jeu
}


